package edu.berkeley.eas.enrollment.dao.sreservation;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.SqlOutParameter;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.SqlReturnResultSet;
import org.springframework.jdbc.object.StoredProcedure;

import edu.berkeley.eas.enrollment.util.StrUtils;

import static edu.berkeley.eas.enrollment.constants.Constants.BLANK;
import static edu.berkeley.eas.enrollment.constants.Constants.FULL_LIMITED;
import static edu.berkeley.eas.enrollment.constants.Constants.RESULT_SET_PARAM;
import static edu.berkeley.eas.enrollment.constants.Constants.YES;

/**
 * SectionReservationsProc provides processing to call SPROC for getting
 * SectionReservations from the database.
 * <p/>
 * procName: sect201
 *
 * @author khensel@berkeley.edu
 * @version %I%, %G%
 */
public class SectionReservationsProc extends StoredProcedure {

    private static final Logger LOGGER = LoggerFactory.getLogger(SectionReservationsProc.class);

    private static final String CCN_PARAM = "LINK_CCN";
    private static final String TERM_YEAR_PARAM = "LINK_TERM_YR";
    private static final String TERM_CODE_PARAM = "LINK_TERM_CD";

    protected static final String ERROR_PARAM = "ERROR_1";
    protected static final String SQL_RETURN_PARAM = "SQL_RETURN";

    @SuppressWarnings("rawtypes")
    public SectionReservationsProc(DataSource dataSource, String procName) {
        super(dataSource, procName);
        declareParameter(new SqlParameter(CCN_PARAM, Types.INTEGER));
        declareParameter(new SqlParameter(TERM_YEAR_PARAM, Types.CHAR));
        declareParameter(new SqlParameter(TERM_CODE_PARAM, Types.CHAR));
        declareParameter(new SqlOutParameter(ERROR_PARAM, Types.CHAR));
        declareParameter(new SqlOutParameter(SQL_RETURN_PARAM, Types.INTEGER));
        declareParameter(new SqlReturnResultSet(RESULT_SET_PARAM, new RowMapper() {
            public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
                SectionReservationsRowMapper mapper = new SectionReservationsRowMapper();
                mapper.setSubTermCode(rs.getString("TERM_SUB_CODE"));
                mapper.setCcn(StrUtils.padCCN(Integer.toString(rs.getInt("COURSE_CONTROL_NUMBER"))));
                mapper.setDepartmentCode(StrUtils.trim(rs.getString("DEPARTMENT_CODE")));
                mapper.setDepartmentName(StrUtils.trim(rs.getString("DEPARTMENT_NAME")));
                mapper.setPrefix(StrUtils.trim(StrUtils.trim(rs.getString("PREFIX_NUM"))));
                mapper.setRootNumber(StrUtils.trim(rs.getString("ROOT_NUMBER")));
                mapper.setSuffixOne(StrUtils.trim(rs.getString("SUFFIX_NUMBER_ONE")));
                mapper.setSuffixTwo(StrUtils.trim(rs.getString("SUFFIX_NUMBER_TWO")));
                mapper.setPrimarySectionFlag(StrUtils.trim(rs.getString("PRIMARY_SECTION_FLAG")));
                mapper.setSectionNumber(StrUtils.trim(rs.getString("SECTION_NUMBER")));
                mapper.setInstructionalFormat(StrUtils.trim(rs.getString("INSTRUCTIONAL_FORMAT")));
                mapper.setSeatRestrictionCode(StrUtils.trim(rs.getString("SEAT_RESTRICTION_CODE")));
                mapper.setInstructorApprovalRequired(StrUtils.toBoolean(rs.getString("INSTRUCTOR_APPROVAL_FLAG")));
                String fullFlag = StrUtils.trim(rs.getString("FULL_FLAG"));
                mapper.setFull(fullFlag.equalsIgnoreCase(FULL_LIMITED)
                        || fullFlag.equalsIgnoreCase(YES));
                mapper.setEnrollCount(Integer.parseInt(rs.getString("ENROLL_COUNT").trim()));
                mapper.setEnrollLimit(Integer.parseInt(rs.getString("ENROLL_LIMIT").trim()));
                mapper.setWaitListCode(rs.getString("WAITLST_CD"));
                mapper.setWaitListCount(Integer.parseInt(rs.getString("WAITLIST_COUNT").trim()));
                mapper.setWaitListLimit(Integer.parseInt(rs.getString("WAITLIST_LIMIT").trim()));
                mapper.setWaitListProcessCode(StrUtils.trim(rs.getString("WAITLIST_PROCESS_CODE")));
                String waitStatus = StrUtils.trim(rs.getString("WAITLIST_STATUS_CODE"));
                mapper.setWaitListStatusCode(waitStatus.equals(BLANK)
                        || waitStatus.equalsIgnoreCase(YES));
                mapper.setUgGradCode(StrUtils.trim(rs.getString("UNDER_GRAD_FLAG")));
                mapper.setCollegeCodeOne(StrUtils.trim(rs.getString("COLLEGE_CODE_ONE")));
                mapper.setCollegeNameOne(StrUtils.trim(rs.getString("COLLEGE_NAME_ONE")));
                mapper.setCollegeCodeTwo(StrUtils.trim(rs.getString("COLLEGE_CODE_TWO")));
                mapper.setCollegeNameTwo(StrUtils.trim(rs.getString("COLLEGE_NAME_TWO")));
                mapper.setMajorCodeOne(StrUtils.trim(rs.getString("MAJOR_CODE_ONE")));
                mapper.setMajorNameOne(StrUtils.trim(rs.getString("MAJOR_NAME_ONE")));
                mapper.setMajorCodeTwo(StrUtils.trim(rs.getString("MAJOR_CODE_TWO")));
                mapper.setMajorNameTwo(StrUtils.trim(rs.getString("MAJOR_NAME_TWO")));
                mapper.setMajorCodeThree(StrUtils.trim(rs.getString("MAJOR_CODE_THREE")));
                mapper.setMajorNameThree(StrUtils.trim(rs.getString("MAJOR_NAME_THREE")));
                mapper.setEducationLevelOne(StrUtils.trim(rs.getString("EDUCATION_LEVEL_ONE")));
                mapper.setEducationLevelOneDesc(StrUtils.trim(rs.getString("EDUC_LVL1_DESC")));
                mapper.setEducationLevelTwo(StrUtils.trim(rs.getString("EDUCATION_LEVEL_TWO")));
                mapper.setEducationLevelTwoDesc(StrUtils.trim(rs.getString("EDUC_LVL2_DESC")));
                mapper.setEducationLevelThree(StrUtils.trim(rs.getString("EDUCATION_LEVEL_THREE")));
                mapper.setEducationLevelThreeDesc(StrUtils.trim(rs.getString("EDUC_LVL3_DESC")));
                mapper.setEducationLevelType(StrUtils.trim(rs.getString("EDUCATION_LEVEL_TYPE")));
                mapper.setRegStatusOne(StrUtils.trim(rs.getString("REG_STATUS_ONE")));
                mapper.setRegStatusOneDesc(StrUtils.trim(rs.getString("REG_STA_CD_DESC1")));
                mapper.setRegStatusTwo(StrUtils.trim(rs.getString("REG_STATUS_TWO")));
                mapper.setRegStatusTwoDesc(StrUtils.trim(rs.getString("REG_STA_CD_DESC2")));
                mapper.setEnglishCompRequiredCode(StrUtils.trim(rs.getString("ENGLISH_COMP_REQUIRED_CD")));
                mapper.setAdmitSpecialProgramCodeOne(StrUtils.trim(rs.getString("ADMIT_SPECIAL_PROGRAM_CODE_ONE")));
                mapper.setAdmitSpecialProgramCodeTwo(StrUtils.trim(rs.getString("ADMIT_SPECIAL_PROGRAM_CODE_TWO")));
                mapper.setUcRequirementOne(StrUtils.trim(rs.getString("UC_REQUIREMENT_ONE")));
                mapper.setUcRequirementOneDesc(StrUtils.trim(rs.getString("UC_REQ1_DESC")));
                mapper.setUcRequirementTwo(StrUtils.trim(rs.getString("UC_REQUIREMENT_TWO")));
                mapper.setUcRequirementTwoDesc(StrUtils.trim(rs.getString("UC_REQ2_DESC")));
                mapper.setPriorityPattern(StrUtils.trim(rs.getString("PRTY_PATTERN_ONE")));
                return mapper;
            }
        }));
        compile();
    }

    public Map<String, Object> execute(int ccn, String termYear, String termCode) {
        LOGGER.debug("+++ in SectionReservationsProc.execute({}, {}, {})",
                Integer.toString(ccn), termYear, termCode);
        Map<String, Object> inputs = new HashMap<String, Object>();
        inputs.put(CCN_PARAM, ccn);
        inputs.put(TERM_YEAR_PARAM, termYear);
        inputs.put(TERM_CODE_PARAM, termCode);
        return super.execute(inputs);
    }
}